
package gui;

import java.awt.Cursor;
import java.awt.Point;
import main.MyProblem;
import main.MyState;
import problem.Action;
import problem.Solver;
import strategy.*;

public class Main extends javax.swing.JFrame {

    private int selected = 0;
    private Solver s;
    public final int[][] grid = new int[25][];
    private Action actions[] ;
    private Point start = new Point(0, 0);
    private Point end = new Point(24, 24);

    public Main() {
        initComponents();
        myBoard.setCursor(new Cursor(Cursor.HAND_CURSOR));
        jComboBoxStrategy.setModel(new javax.swing.DefaultComboBoxModel(
                new Strategy[]{
                    new BreadthFirstSearch(),
                    new DepthFirstSearch(),
                    new AStarSearch(),
                    new IterativeDeepeningSearch(),
                    new RecursiveBestFirstSearch()
                }));
        
        grid[24] = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0};
        grid[23] = new int[]{0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        grid[22] = new int[]{0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        grid[21] = new int[]{0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        grid[20] = new int[]{0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0};
        grid[19] = new int[]{0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0};
        grid[18] = new int[]{0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1};
        grid[17] = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1};
        grid[16] = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0};
        grid[15] = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0};
        grid[14] = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0};
        grid[13] = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        grid[12] = new int[]{0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        grid[11] = new int[]{0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        grid[10] = new int[]{0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0};
        grid[9] = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0};
        grid[8] = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0};
        grid[7] = new int[]{0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0};
        grid[6] = new int[]{0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0};
        grid[5] = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1, 1};
        grid[4] = new int[]{0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1};
        grid[3] = new int[]{0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 0};
        grid[2] = new int[]{0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0};
        grid[1] = new int[]{0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0};
        grid[0] = new int[]{0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0};

        this.myBoard.setGrid(grid);


        this.setAlwaysOnTop(true);
        this.setTitle("Maze [ author : Ahmad Dehnavi && Peyman Dehghan]");
        this.setLocationRelativeTo(null);


        actions = new Action[4];
        
        //bottom
        this.actions[0] = new Action<MyState>() {

            @Override
            public MyState getNextState(MyState currentState) {
                return new MyState(currentState.getX(), currentState.getY() - 1);
            }

            @Override
            public float getCost() {
                return 1;
            }
        };
        //left
        this.actions[1] = new Action<MyState>() {

            @Override
            public MyState getNextState(MyState currentState) {
                return new MyState(currentState.getX() - 1, currentState.getY());
            }
             @Override
            public float getCost() {
                return 1;
            }
        };
        //right
        this.actions[2] = new Action<MyState>() {

            @Override
            public MyState getNextState(MyState currentState) {
                return new MyState(currentState.getX() + 1, currentState.getY());
            }
             @Override
            public float getCost() {
                return 1;
            }
        };
        //top
        this.actions[3] = new Action<MyState>() {

            @Override
            public MyState getNextState(MyState currentState) {
                return new MyState(currentState.getX(), currentState.getY() + 1);
            }
             @Override
            public float getCost() {
                return 1;
            }
        };
        
/* this.actions[4] = new Action<MyState>() {

            @Override
            public MyState getNextState(MyState currentState) {
                return new MyState(currentState.getX()+1, currentState.getY() + 1);
            }
             @Override
            public float getCost() {
                return (float) Math.sqrt(2);
            }
        };

        this.actions[5] = new Action<MyState>() {

            @Override
            public MyState getNextState(MyState currentState) {
                return new MyState(currentState.getX()-1, currentState.getY() - 1);
            }
             @Override
            public float getCost() {
                return (float) Math.sqrt(2);
            }
        };
         this.actions[6] = new Action<MyState>() {

            @Override
            public MyState getNextState(MyState currentState) {
                return new MyState(currentState.getX()-1, currentState.getY() + 1);
            }
             @Override
            public float getCost() {
                return (float) Math.sqrt(2);
            }
        };

        this.actions[7] = new Action<MyState>() {

            @Override
            public MyState getNextState(MyState currentState) {
                return new MyState(currentState.getX()+1, currentState.getY() - 1);
            }
             @Override
            public float getCost() {
                return (float) Math.sqrt(2);
            }
        };
*/
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        myBoard = new Board.board();
        jPanel1 = new javax.swing.JPanel();
        labelBlocked = new java.awt.Label();
        labelClear = new java.awt.Label();
        labelTreeNode = new java.awt.Label();
        labelFring = new java.awt.Label();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        labelPath = new java.awt.Label();
        jLabel5 = new javax.swing.JLabel();
        jButtonFindPath = new javax.swing.JButton();
        jButtonSetEnd = new javax.swing.JButton();
        jButtonSetStart = new javax.swing.JButton();
        jComboBoxStrategy = new javax.swing.JComboBox();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabelMemoryCompiexity = new javax.swing.JLabel();
        jLabelTimeCompiexity = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        myBoard.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                myBoardMousePressed(evt);
            }
        });
        myBoard.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseDragged(java.awt.event.MouseEvent evt) {
                myBoardMouseDragged(evt);
            }
        });

        javax.swing.GroupLayout myBoardLayout = new javax.swing.GroupLayout(myBoard);
        myBoard.setLayout(myBoardLayout);
        myBoardLayout.setHorizontalGroup(
            myBoardLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 550, Short.MAX_VALUE)
        );
        myBoardLayout.setVerticalGroup(
            myBoardLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );

        labelBlocked.setBackground(new java.awt.Color(0, 0, 0));
        labelBlocked.setCursor(new java.awt.Cursor(java.awt.Cursor.CROSSHAIR_CURSOR));
        labelBlocked.setText("      ");
        labelBlocked.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                labelBlockedMouseClicked(evt);
            }
        });

        labelClear.setBackground(new java.awt.Color(255, 255, 255));
        labelClear.setCursor(new java.awt.Cursor(java.awt.Cursor.CROSSHAIR_CURSOR));
        labelClear.setText("      ");
        labelClear.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                labelClearMouseClicked(evt);
            }
        });

        labelTreeNode.setBackground(new java.awt.Color(200, 200, 200));
        labelTreeNode.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        labelTreeNode.setText("      ");

        labelFring.setBackground(new java.awt.Color(20, 200, 20));
        labelFring.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        labelFring.setText("      ");

        jLabel1.setText("Clear");

        jLabel2.setText("Tree Node");

        jLabel3.setText("Blocked");

        jLabel4.setText("Fring");

        labelPath.setBackground(new java.awt.Color(50, 200, 250));
        labelPath.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        labelPath.setText("      ");

        jLabel5.setText("Path");

        jButtonFindPath.setText("Find Path");
        jButtonFindPath.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonFindPathActionPerformed(evt);
            }
        });

        jButtonSetEnd.setText("Set End Point");
        jButtonSetEnd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSetEndActionPerformed(evt);
            }
        });

        jButtonSetStart.setText("Set Start Point");
        jButtonSetStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSetStartActionPerformed(evt);
            }
        });

        jLabel7.setText("Space Compiexity :");

        jLabel8.setText("Time Compiexity :");

        jLabelMemoryCompiexity.setText("0");

        jLabelTimeCompiexity.setText("0  ");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(jComboBoxStrategy, javax.swing.GroupLayout.PREFERRED_SIZE, 112, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButtonSetStart, javax.swing.GroupLayout.PREFERRED_SIZE, 112, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButtonSetEnd, javax.swing.GroupLayout.PREFERRED_SIZE, 112, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButtonFindPath, javax.swing.GroupLayout.PREFERRED_SIZE, 112, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(10, 10, 10))
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(10, 10, 10)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2)
                    .addComponent(jLabel1)
                    .addComponent(jLabel5, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 71, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, 46, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelFring, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 32, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(labelTreeNode, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 31, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(labelClear, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(labelBlocked, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(labelPath, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 32, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabelTimeCompiexity, javax.swing.GroupLayout.PREFERRED_SIZE, 113, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel8)))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel7)
                            .addComponent(jLabelMemoryCompiexity, javax.swing.GroupLayout.PREFERRED_SIZE, 113, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(jLabel1)
                    .addComponent(labelClear, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(labelBlocked, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(jLabel2)
                    .addComponent(labelTreeNode, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(labelFring, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel4))
                .addGap(12, 12, 12)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(jLabel5)
                    .addComponent(labelPath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(21, 21, 21)
                .addComponent(jButtonSetStart, javax.swing.GroupLayout.PREFERRED_SIZE, 30, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButtonSetEnd, javax.swing.GroupLayout.PREFERRED_SIZE, 30, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jComboBoxStrategy, javax.swing.GroupLayout.PREFERRED_SIZE, 30, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jButtonFindPath, javax.swing.GroupLayout.PREFERRED_SIZE, 31, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(33, 33, 33)
                .addComponent(jLabel7)
                .addGap(8, 8, 8)
                .addComponent(jLabelMemoryCompiexity)
                .addGap(23, 23, 23)
                .addComponent(jLabel8)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabelTimeCompiexity)
                .addContainerGap(34, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(myBoard, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(2, 2, 2))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(myBoard, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void myBoardMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_myBoardMousePressed

        try {
            int width = myBoard.getWidth(), height = myBoard.getHeight();
            int w = grid.length;
            int h = grid.length;

            int i = evt.getX() / (width / w);
            int j = -((evt.getY() / (height / h)) + 1 - h);

            Point temp = new Point(i, j);
            if (start.equals(temp)) {
                selected = -1;
            } else if (end.equals(temp)) {
                selected = -2;
            }

            if (selected < 0) {
                if (this.grid[i][j] == MyProblem.BLOCKED_TILE) {
                    return;
                }
                if (selected == -1) {
                    this.start = temp;
                    this.myBoard.setStartPoint(temp);

                } else if (selected == -2) {
                    this.end = temp;
                    this.myBoard.setEndPoint(temp);

                }
                return;
            }

            if (evt.getButton() == 3) {
                grid[i][j] = this.selected == MyProblem.BLOCKED_TILE ? MyProblem.CLEAR_TILE : MyProblem.BLOCKED_TILE;
            } else {
                grid[i][j] = this.selected;
            }

            this.repaint();
        } catch (Exception e) {
        }

    }//GEN-LAST:event_myBoardMousePressed

    private void myBoardMouseDragged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_myBoardMouseDragged
        myBoardMousePressed(evt);
    }//GEN-LAST:event_myBoardMouseDragged

    private void labelClearMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_labelClearMouseClicked
        this.selected = MyProblem.CLEAR_TILE;
    }//GEN-LAST:event_labelClearMouseClicked

    private void labelBlockedMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_labelBlockedMouseClicked
        this.selected = MyProblem.BLOCKED_TILE;
    }//GEN-LAST:event_labelBlockedMouseClicked

    private void jButtonFindPathActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonFindPathActionPerformed

        s = new Solver();
        s.setProblem(new MyProblem(new MyState(start.x, start.y), new MyState(end.x, end.y), actions, grid)).reset();
        s.setStrategy((Strategy)jComboBoxStrategy.getSelectedItem());
        s.solve();
        this.repaint();
        this.jLabelMemoryCompiexity.setText(s.getStrategy().getMaximumNumberOfTreeNode()+"");
        this.jLabelTimeCompiexity.setText(s.getStrategy().getNumberOfProcessedNode()+"");
    }//GEN-LAST:event_jButtonFindPathActionPerformed

    private void jButtonSetEndActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSetEndActionPerformed
        selected = -2;
    }//GEN-LAST:event_jButtonSetEndActionPerformed

    private void jButtonSetStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSetStartActionPerformed
        selected = -1;
    }//GEN-LAST:event_jButtonSetStartActionPerformed

    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /*
         * Set the Nimbus look and feel
         */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /*
         * If Nimbus (introduced in Java SE 6) is not available, stay with the
         * default look and feel. For details see
         * http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(Main.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(Main.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(Main.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(Main.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /*
         * Create and display the form
         */
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new Main().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonFindPath;
    private javax.swing.JButton jButtonSetEnd;
    private javax.swing.JButton jButtonSetStart;
    private javax.swing.JComboBox jComboBoxStrategy;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabelMemoryCompiexity;
    private javax.swing.JLabel jLabelTimeCompiexity;
    private javax.swing.JPanel jPanel1;
    private java.awt.Label labelBlocked;
    private java.awt.Label labelClear;
    private java.awt.Label labelFring;
    private java.awt.Label labelPath;
    private java.awt.Label labelTreeNode;
    private Board.board myBoard;
    // End of variables declaration//GEN-END:variables

}


