package strategy;

import java.util.Iterator;
import java.util.LinkedList;
import problem.Action;
import problem.Node;
import problem.Problem;
import problem.State;

public class AStarSearch extends Strategy {

    private LinkedList<Node> openList = new LinkedList<>();
    private LinkedList<Node> closeList = new LinkedList<>();
    private Node goalNode;
    private Problem problem;

    private int getF(Node n) {
        return n.getDepth() + problem.calculateHeuristic(n);
    }

    private void addChild(Node newNode) {
       //check open list
        int f = this.getF(newNode);
        if (problem.isFring(newNode)) {
            Iterator<Node> it = openList.iterator();
            while (it.hasNext()) {
                Node alreadyExistNode = it.next();
                if (newNode.equals(alreadyExistNode)) {
                    if (f > this.getF(alreadyExistNode)) {
                        return;//discard
                    } else {
                        this.removeFromOpenList(alreadyExistNode);
                        this.decrementNumberOfNode();
                    }
                }
            }
        }
        
        //check close list
        if (problem.isVisited(newNode)) {
            Iterator<Node> it = closeList.iterator();
            while (it.hasNext()) {
                Node alreadyExistNode = it.next();
                if (newNode.equals(alreadyExistNode)) {
                    if (f > this.getF(alreadyExistNode)) {
                        return;//discard new Node;
                    } else {
                        this.removeFromCloseList(alreadyExistNode);
                        this.decrementNumberOfNode();
                    }
                }
            }
        }
        
        this.incrementNumberOfNode();
        this.addToOpenList(newNode);
    }

    private void addToOpenList(Node n) {
        problem.fring(n);
        openList.add(n);
    }

    private void addToCloseList(Node n) {
        problem.visited(n);
        closeList.add(n);
     }

    private void removeFromOpenList(Node n) {
        problem.Clear(n);
        openList.remove(n);
    }

    private Node removeFromOpenList() {
        int lowest = Integer.MAX_VALUE;
        Node lowestNode = null;
        int i = -1, lowestIndex = 0;
        for (Iterator<Node> it = openList.iterator(); it.hasNext();) {
            i++;
            Node n = it.next();
            if (this.getF(n) < lowest) {
                lowestNode = n;
                lowestIndex = i;
                lowest = this.getF(n);
            }
        }

        openList.remove(lowestIndex);
        problem.Clear(lowestNode);
        
        return lowestNode;
    }

    private void removeFromCloseList(Node n) {
        problem.Clear(n);
        closeList.remove(n);
    }

    @Override
    public Node Solve(Problem problem) {
        this.resetCalculatedComplexity();
        
        this.problem = problem;

        openList.clear();
        closeList.clear();

        Action[] action = problem.getActions();

        this.addToOpenList(new Node(problem.getInitState()));
        goalNode = new Node(problem.getGoalState());

        while (!openList.isEmpty()) {

            Node parent = this.removeFromOpenList();

            //goal check.
            if (parent.equals(goalNode)) {
                return parent;
            }
            this.addToCloseList(parent);

            
            //Expand
            for (Action a : action) {
                State s = a.getNextState(parent.getState());

                if (problem.isValidState(s)) {
                    Node node = new Node(s, parent,a);
                    if (problem.isClear(node)) {
                        this.addChild(node);
                    }
                }

            }
        }
        return null;
    }

    @Override
    public String toString() {
        return "A Star";
    }
    
    
}
