package strategy;

import java.util.Iterator;
import java.util.LinkedList;
import problem.Action;
import problem.Node;
import problem.Problem;
import problem.State;

public class RecursiveBestFirstSearch extends Strategy {

    private Problem problem;
    private Action[] action;

    @Override
    public Node Solve(Problem problem) {
        this.resetCalculatedComplexity();

        this.problem = problem;
        this.action = problem.getActions();

        return RBFS(new RBFSNode(problem.getInitState()), Integer.MAX_VALUE).getNode();
    }

    private Result RBFS(RBFSNode node, float fLimit) {

        if (problem.isGoal(node)) {
            this.decrementNumberOfNode();
            return new Result().solotion(node);
        }

        problem.visited(node);

        LinkedList<RBFSNode> succ = new LinkedList<>();

        for (Action a : action) {
            State state = a.getNextState(node.getState());
            if (problem.isValidState(state)) {
                RBFSNode s = new RBFSNode(state, node, a);
                
                if (problem.isClear(s)) {

                    s.setF(Math.max(s.getG() + problem.calculateHeuristic(s), node.getF()));
                    succ.add(s);
                    problem.fring(s);
                    this.incrementNumberOfNode();

                }
            }
        }

        if (succ.isEmpty()) {
            this.decrementNumberOfNode();
            return new Result().failure(Integer.MAX_VALUE);
        }

        while (true) {

            RBFSNode[] temp = getBest(succ);
            RBFSNode best = temp[0];

            if (best.getF() > fLimit) {
                this.decrementNumberOfNode();
                return new Result().failure(best.getF());
            }
            
            RBFSNode alt = temp[1];

            Result r = RBFS(best, Math.min(fLimit, alt.getF()));

            if (!r.isFailure()) {
                this.decrementNumberOfNode();
                return r;
            }

            best.setF(r.getBest());

        }

    }

    private RBFSNode[] getBest(LinkedList<RBFSNode> succ) {
        RBFSNode best = succ.peek();
        RBFSNode alt = best;

        for (Iterator<RBFSNode> it = succ.iterator(); it.hasNext();) {
            RBFSNode node = it.next();
            if (best.getF() > node.getF()) {
                alt = best;
                best = node;
            }
        }

        return new RBFSNode[]{best, alt};
    }

    private class Result {

        private Node node;
        private boolean failure = true;
        private float best;

        public float getBest() {
            return best;
        }

        public void setBest(int best) {
            this.best = best;
        }

        public Node getNode() {
            return node;
        }

        public void setNode(Node node) {
            this.node = node;
        }

        public Result solotion(Node r) {
            this.node = r;
            this.failure = false;
            return this;
        }

        public Result failure(float best) {
            this.failure = true;
            this.best = best;
            return this;
        }

        public boolean isSolotion() {
            return !this.failure;
        }

        public boolean isFailure() {
            return this.failure;
        }
    }

    private class RBFSNode extends Node {

        private float f = 0;

        public RBFSNode(State state) {
            super(state);
        }

        public RBFSNode(State state, Node parent, Action action) {
            super(state, parent, action);
        }

        public float getF() {
            return f;
        }

        public void setF(float f) {
            this.f = f;
        }
    }

    @Override
    public String toString() {
        return "Recursive Best First";
    }
    
    
}
